#################################################
### function to compute bayes/gpq boundaries
### based on effect size and dual additive
### test boundaries.  quantiles and bayes
### probabilities of exceeding specified boundaries
### are returned
###
bgpq<-function(xR,xB,cL,cU,alphaL,alphaU,nsim=10000){

	### sample moments
	xbarR<-mean(xR)
	xbarB<-mean(xB)
	xvarR<-var(xR)
	xvarB<-var(xB)
	
	### simulation from parameter posterior distributions
	simsigR<- sqrt( (nR-1)*xvarR/rchisq(nsim,nR-1) )
	simsigB<- sqrt( (nB-1)*xvarB/rchisq(nsim,nB-1) )
	simmuR<-rnorm(nsim,xbarR,simsigR/sqrt(nR))
	simmuB<-rnorm(nsim,xbarB,simsigB/sqrt(nB))
	
	### functions of the parameters
	
	simlambda<- (simmuB-simmuR)/simsigR
	### fda asymptotic-approach parameters
	simtL<-(simmuB-simmuR)+cL*simsigR
	simtU<-(simmuB-simmuR)-cU*simsigR
	
	### quantiles of simulated functions of the parameters
	lamQ<-quantile(simlambda,c(alphaL,1-alphaU))
	tQ<-c(quantile(simtL,alphaL),quantile(simtU,1-alphaU))
	
	### probabilities of exceeding hypothesized boundaries
	### from fda et al
	plam<-c(mean(simlambda<= -cL),mean(simlambda>=cU))
	ptstat<-c(mean(simtL<=0),mean(simtU>=0))
	
	return(list(lamQ=lamQ,tQ=tQ,plam=plam,ptstat=ptstat))
}


#############################################################################
#### testing based on one-sided p-values and quantiles

### test boundaries and nominal levels
cL<- 1.5
cU<- 1.5
alphaL<-0.05
alphaU<-0.05

### population parameter values
muR<- 0
sigR<-1
sigB<-sqrt(1)
muB<- muR-cL*sigR  ### lower null value

### sample size
nR<-10
nB<-10

### simulate data
set.seed(12357)
nrep<-10000 
declamQ<-rep(NA,nrep)
decplam<-rep(NA,nrep)
dectQ<-rep(NA,nrep)
decptstat<-rep(NA,nrep)
lowTaillamQ<-rep(NA,nrep)
lowTailplam<-rep(NA,nrep)
lowTailtQ<-rep(NA,nrep)
lowTailptstat<-rep(NA,nrep)
for(i in 1:nrep){
	xR<-rnorm(nR,muR,sigR)
	xB<-rnorm(nB,muB,sigB)
	
	out<-bgpq(xR,xB,cL,cU,alphaL,alphaU)
	declamQ[i]<-(out$lamQ[1]>= -cL) && (out$lamQ[2]<=cU)
	decplam[i]<-(out$plam[1]<=alphaL) && (out$plam[2]<=alphaU)
	dectQ[i]<-(out$tQ[1]>=0 && out$tQ[2]<=0)
	decptstat[i]<-(out$ptstat[1]<=alphaL) && (out$ptstat[2]<=alphaU)
	
	lowTaillamQ[i]<-(out$lamQ[1]>= -cL) 
	lowTailplam[i]<-(out$plam[1]<=alphaL) 
	lowTailtQ[i]<-(out$tQ[1]>=0 )
	lowTailptstat[i]<-(out$ptstat[1]<=alphaL) 
}

### proportion of samples in which the null rejected
mean(declamQ)
mean(decplam)
mean(dectQ)
mean(decptstat)

### count of number of different decisions based on the different
### decision criteria
sum(declamQ!=decptstat)


##### proportion of samples in which the lower null value is rejected
mean(lowTaillamQ)
mean(lowTailplam)
mean(lowTailtQ)
mean(lowTailptstat)


### simulation SE
sqrt(alphaL*(1-alphaL)/nrep)
sqrt(alphaU*(1-alphaU)/nrep)


